import { DeliveryMethod } from "@shopify/shopify-api";
import { connection } from "./database/db.js";
import axios from "axios";
import dotenv from "dotenv";

dotenv.config();
/**
 * @type {{[key: string]: import("@shopify/shopify-api").WebhookHandler}}
 */
export default {
  /**
   * Customers can request their data from a store owner. When this happens,
   * Shopify invokes this privacy webhook.
   *
   * https://shopify.dev/docs/apps/webhooks/configuration/mandatory-webhooks#customers-data_request
   */
  CUSTOMERS_DATA_REQUEST: {
    deliveryMethod: DeliveryMethod.Http,
    callbackUrl: "/api/webhooks",
    callback: async (topic, shop, body, webhookId) => {
      const payload = JSON.parse(body);
      // Payload has the following shape:
      // {
      //   "shop_id": 954889,
      //   "shop_domain": "{shop}.myshopify.com",
      //   "orders_requested": [
      //     299938,
      //     280263,
      //     220458
      //   ],
      //   "customer": {
      //     "id": 191167,
      //     "email": "john@example.com",
      //     "phone": "555-625-1199"
      //   },
      //   "data_request": {
      //     "id": 9999
      //   }
      // }
    },
  },

  /**
   * Store owners can request that data is deleted on behalf of a customer. When
   * this happens, Shopify invokes this privacy webhook.
   *
   * https://shopify.dev/docs/apps/webhooks/configuration/mandatory-webhooks#customers-redact
   */
  CUSTOMERS_REDACT: {
    deliveryMethod: DeliveryMethod.Http,
    callbackUrl: "/api/webhooks",
    callback: async (topic, shop, body, webhookId) => {
      const payload = JSON.parse(body);
      // Payload has the following shape:
      // {
      //   "shop_id": 954889,
      //   "shop_domain": "{shop}.myshopify.com",
      //   "customer": {
      //     "id": 191167,
      //     "email": "john@example.com",
      //     "phone": "555-625-1199"
      //   },
      //   "orders_to_redact": [
      //     299938,
      //     280263,
      //     220458
      //   ]
      // }
    },
  },

  /**
   * 48 hours after a store owner uninstalls your app, Shopify invokes this
   * privacy webhook.
   *
   * https://shopify.dev/docs/apps/webhooks/configuration/mandatory-webhooks#shop-redact
   */
  SHOP_REDACT: {
    deliveryMethod: DeliveryMethod.Http,
    callbackUrl: "/api/webhooks",
    callback: async (topic, shop, body, webhookId) => {
      const payload = JSON.parse(body);
      // Payload has the following shape:
      // {
      //   "shop_id": 954889,
      //   "shop_domain": "{shop}.myshopify.com"
      // }
    },
  },

  ORDERS_PAID: {
    deliveryMethod: DeliveryMethod.Http,
    callbackUrl: "/api/webhooks",
    callback: async (topic, shop, body, webhookId) => {
      const payload = JSON.parse(body);

      console.log(payload.line_items[0].properties, "payload 0");

      const isProtectMyOrder = payload.line_items.some((lineItem) => {
        return lineItem.properties.some(
          (property) => property.name === "_ishidden"
        );
      });
      const insertQuery =
        "INSERT INTO orders (first_name, last_name , order_id, price, currency, email, financial_status, priorityProtection, shop) VALUES (?, ?, ?, ?, ?, ?, ?, ?,?)";
      connection.query(
        insertQuery,
        [
          payload.customer.first_name,
          payload.customer.last_name,
          payload.id,
          payload.current_total_price,
          payload.currency,
          payload.customer.email,
          payload.financial_status,
          isProtectMyOrder,
          shop,
        ],
        (err, results) => {
          if (err) throw err;
          console.log("Data inserted successfully:", results);
        }
      );
      const query = `SELECT accessToken FROM shopify_sessions WHERE shop = "${shop}"`;
      const [rows] = await connection.promise().query(query, [shop]);
      const accessToken = rows.length > 0 ? rows[0].accessToken : null;
      const productId = payload.line_items
        .filter((lineItem) => lineItem.title === "PriorityProtection")

        .map((product) => product.product_id);
      const variantId = payload.line_items.filter((lineItem) => {
        console.log(lineItem, "----item");
        return (
          lineItem.title === "PriorityProtection" &&
          lineItem.name !== "PriorityProtection - 2.99" &&
          lineItem.name !== "PriorityProtection - 2.49" &&
          lineItem.name !== "PriorityProtection - 1.99" &&
          lineItem.name !== "PriorityProtection - 4.00"
        );
      });

      console.log(variantId, "---variantId");
      try {
        if (variantId.length > 0) {
          const deleteVariantResponse = await fetch(
            `https://${shop}/admin/api/${process.env.API_VERSION}/products/${productId}/variants/${variantId[0].variant_id}.json`,
            {
              method: "DELETE",
              headers: {
                "X-Shopify-Access-Token": accessToken,
                "Content-Type": "application/json",
              },
            }
          );
          console.log("deleteVariantResponse", deleteVariantResponse);
          if (deleteVariantResponse.ok) {
            console.log(`Variant with ID ${variantId} deleted successfully.`);
          } else {
            console.error(
              `Failed to delete variant: ${deleteVariantResponse.statusText}`
            );
          }
        }
      } catch (error) {
        console.error("Error occurred while deleting the variant:", error);
      }
    },
  },
  ORDERS_FULFILLED: {
    deliveryMethod: DeliveryMethod.Http,
    callbackUrl: "/api/webhooks",
    callback: async (topic, shop, body, webhookId) => {
      const payload = JSON.parse(body);
      console.log(payload, "payload ORDERS FULFILLED");
      const query = `SELECT * FROM orders WHERE order_id = "${payload.id}" AND shop = "${shop}"`;
      const [rows] = await connection.promise().query(query, [shop]);
      const isPriorityProtect =
        rows.length > 0 ? rows[0].priorityProtection : null;
      const PriorityProtectPrice = payload.line_items
        .filter((lineItem) => {
          return lineItem.properties.some(
            (property) => property.name === "_ishidden"
          );
        })
        .map((lineItem) => lineItem.price);

      console.log(PriorityProtectPrice, "----priority protect");

      if (isPriorityProtect == 1) {
        try {
          const query = `SELECT charge_id FROM recurrings WHERE shop = "${shop}"`;
          const [rows] = await connection.promise().query(query, [shop]);
          const recurringId = rows.length > 0 ? rows[0].charge_id : null;
          if (recurringId != null) {
            const response = await axios.post(
              `${process.env.BASE_URL}/usageCharge`,
              {
                shop: shop,
                description: "test",
                price: PriorityProtectPrice[0],
              },
              {
                headers: {
                  "Content-Type": "application/json",
                },
              }
            );
            console.log(response.data);
          }
        } catch (err) {
          console.log(err, "-=--rertew");
        }
      }
    },
  },
  APP_SUBSCRIPTIONS_UPDATE: {
    deliveryMethod: DeliveryMethod.Http,
    callbackUrl: "/api/webhooks",
    callback: async (topic, shop, body, webhookId) => {
      const payload = JSON.parse(body);
      console.log("App subscription update event triggered");
      console.log("Topic: ", topic);
      console.log("Shop: ", shop);
      console.log("Webhook ID: ", webhookId);
      console.log("Payload APP SUBSCRIPTION: ", payload);
      const startDate = new Date(payload.app_subscription.created_at);
      const end_date = new Date(startDate.getTime() + 30 * 24 * 60 * 60 * 1000);
      if (
        payload &&
        payload.app_subscription.status === "ACTIVE" &&
        topic === "APP_SUBSCRIPTIONS_UPDATE"
      ) {
        const response = await axios.get(
          `${process.env.BASE_URL}/createPriorityProtection?shop=${shop}`,

          {
            headers: {
              "Content-Type": "application/json",
            },
          }
        );
        console.log(response.data, "response.data");
        const productId = response.data.product.id;
        console.log("productId", productId);
        try {
          const chargeId = payload.app_subscription.admin_graphql_api_id
            .split("/")
            .pop();

          // Check if the charge ID already exists for this shop
          const checkQuery = `SELECT * FROM recurrings WHERE  shop ="${shop}"`;
          connection.query(checkQuery, [chargeId, shop], (err, results) => {
            if (err) throw err;
            console.log(results, "results", chargeId, shop);
            if (results.length > 0) {
              // If record exists, perform an update
              const updateQuery = `
              UPDATE recurrings
              SET plan_name = ?, capped_amount = ?, currency = ?, product_id = ?, status = ?, start_date = ?, end_date = ?, charge_id = ?
              WHERE shop = ?
            `;
              connection.query(
                updateQuery,
                [
                  payload.app_subscription.name,
                  payload.app_subscription.capped_amount,
                  payload.app_subscription.currency,
                  productId,
                  payload.app_subscription.status,
                  startDate,
                  end_date,
                  chargeId,
                  shop,
                ],
                (err, results) => {
                  if (err) throw err;
                  console.log("Data updated successfully:", results);
                }
              );
            } else {
              // If record does not exist, perform an insert
              const insertQuery = `
              INSERT INTO recurrings
              (plan_name, capped_amount, charge_id, shop, currency,product_id, status, start_date, end_date)
              VALUES (?, ?, ?, ?, ?, ?, ?, ?,?)
            `;
              connection.query(
                insertQuery,
                [
                  payload.app_subscription.name,
                  payload.app_subscription.capped_amount,
                  chargeId,
                  shop,
                  payload.app_subscription.currency,
                  productId,
                  payload.app_subscription.status,
                  startDate,
                  end_date,
                ],
                (err, results) => {
                  if (err) throw err;
                  console.log("Data inserted successfully:", results);
                }
              );
            }
          });
        } catch (err) {
          console.log(err, "---err");
        }
      }
    },
  },

  APP_UNINSTALLED: {
    deliveryMethod: DeliveryMethod.Http,
    callbackUrl: "/api/webhooks",
    callback: async (topic, shop, body, webhookId) => {
      const payload = JSON.parse(body);
      console.log(payload, "---uninstall");
      if (payload && topic === "APP_UNINSTALLED") {
        try {
          const deleteOrdersQuery = `DELETE FROM orders WHERE shop = "${shop}" `;
          const deleteRecurringQuery = `DELETE FROM recurrings WHERE shop = "${shop}" `;

          connection.query(deleteOrdersQuery, (err, results) => {
            if (err) throw err;
            console.log("Orders table deleted shop data successfully!");
          });

          connection.query(deleteRecurringQuery, (err, results) => {
            if (err) throw err;
            console.log("Recurring table deleted shop successfully!");
          });
        } catch (err) {
          console.log(err, "---err");
        }
      }
    },
  },
};
