// @ts-check
import { join } from "path";
import { readFileSync } from "fs";
import express from "express";
import serveStatic from "serve-static";
import { connection } from "./database/db.js";
import shopify from "./shopify.js";
import axios from "axios";
import PrivacyWebhookHandlers from "./privacy.js";
import path from "path";
import { URL, fileURLToPath } from "url";
import { dirname } from "path";
import cors from "cors";
import fs from "fs";
const PORT = parseInt(
  process.env.BACKEND_PORT || process.env.PORT || "3000",
  10
);

const STATIC_PATH =
  process.env.NODE_ENV === "production"
    ? `${process.cwd()}/frontend/dist`
    : `${process.cwd()}/frontend/`;

const app = express();
app.use(cors());

connection.connect((err) => {
  if (err) throw err;
  console.log("Connected!");
});
// Set up Shopify authentication and webhook handling
app.get(shopify.config.auth.path, shopify.auth.begin());
app.get(
  shopify.config.auth.callbackPath,
  shopify.auth.callback(),
  shopify.redirectToShopifyOrAppRoot()
);
app.post(
  shopify.config.webhooks.path,
  shopify.processWebhooks({ webhookHandlers: PrivacyWebhookHandlers })
);

// If you are adding routes outside of the /api path, remember to
// also add a proxy rule for them in web/frontend/vite.config.js

app.use("/api/*", shopify.validateAuthenticatedSession());

app.use(express.json());
const __dirname = new URL(".", import.meta.url).pathname;

app.use("/assets", express.static(path.join(__dirname, "web/frontend/assets")));
app.get("/assets/priority-protection.png", (req, res) => {
  res.sendFile(
    path.join(__dirname, "web/frontend/assets/priority-protection.png")
  );
});

app.get("/api/products/count", async (_req, res) => {
  const countData = await shopify.api.rest.Product.count({
    session: res.locals.shopify.session,
  });
  res.status(200).send(countData);
});
app.post("/api/webhooks", async (_req, res) => {
  try {
    return res.status(200).set("Content-Type", "text/html");
  } catch {
    return res.status(401).set("Content-Type", "text/html");
  }
});
app.get("/createPriorityProtection", async (req, res) => {
  const shop = req.query.shop;
  const query = `SELECT accessToken FROM shopify_sessions WHERE shop = "${shop}"`;
  const [rows] = await connection.promise().query(query, [shop]);
  const accessToken = rows.length > 0 ? rows[0].accessToken : null;

  const productData = {
    title: "PriorityProtection",
    handle: `${process.env.PRODUCT_HANDLE}`,
    vendor: shop,
    published_scope: "global",
    images: [
      {
        src: `${process.env.BASE_URL}/web/frontend/assets/priority-protection.png`,
      },
    ],
    metafields: [
      {
        key: "hidden",
        value: 1,
        namespace: "seo",
        type: "integer",
      },
    ],
    variants: [
      {
        option1: "1.99",
        price: "$1.99",
      },
      {
        option1: "2.49",
        price: "$2.49",
      },
      {
        option1: "2.99",
        price: "$2.99",
      },
      {
        option1: "4.00",
        price: "$4.00",
      },
    ],
  };

  const collectionData = {
    title: "All Products Excluded Priority Protection",
    body_html:
      "<p>Collection to specify all products not offered by Priority Protection. This collection or one like it should be used to exclude the Priority Protection product from your online store. To prevent the Priority Protection item from showing up on your store, use the query here in this collection. If you don't have an 'all' collection set up, you can use this one directly (make sure to update this description!). Otherwise, copy the approach taken here to exclude the Priority Protection product from other collections you have in your store.</p>",
    rules: [
      {
        column: "title",
        relation: "not_contains",
        condition: "Priority",
      },
      {
        column: "title",
        relation: "not_contains",
        condition: "Protection",
      },
    ],
    disjunctive: true,
  };

  const productUrl = `https://${shop}/admin/api/${process.env.API_VERSION}/products.json`;
  const checkProductUrl = `https://${shop}/admin/api/${
    process.env.API_VERSION
  }/products.json?title=${encodeURIComponent(productData.title)}`;
  const collectionUrl = `https://${shop}/admin/api/${process.env.API_VERSION}/smart_collections.json`;
  const checkCollectionUrl = `${collectionUrl}?title=${encodeURIComponent(
    collectionData.title
  )}`;

  try {
    let productResponseData = null;
    let collectionResponseData = null;
    let productExists = false;
    let collectionExists = false;

    // Check if the product already exists
    const checkProductResponse = await fetch(checkProductUrl, {
      method: "GET",
      headers: {
        "X-Shopify-Access-Token": accessToken,
        "Content-Type": "application/json",
      },
    });

    if (!checkProductResponse.ok) {
      throw new Error(
        `Error checking product: ${checkProductResponse.statusText}`
      );
    }

    const existingProduct = await checkProductResponse.json();

    if (existingProduct.products.length > 0) {
      productResponseData = existingProduct.products[0];
      productExists = true;
    } else {
      // Create the product if it does not exist
      const productResponse = await fetch(productUrl, {
        method: "POST",
        headers: {
          "X-Shopify-Access-Token": accessToken,
          "Content-Type": "application/json",
        },
        body: JSON.stringify({ product: productData }),
      });
      if (!productResponse.ok) {
        throw new Error(`Error adding product: ${productResponse.statusText}`);
      }

      const productDataResponse = await productResponse.json();
      productResponseData = productDataResponse.product;
    }

    // Check if the collection already exists
    const checkCollectionResponse = await fetch(checkCollectionUrl, {
      method: "GET",
      headers: {
        "X-Shopify-Access-Token": accessToken,
        "Content-Type": "application/json",
      },
    });

    if (!checkCollectionResponse.ok) {
      throw new Error(
        `Error checking collection: ${checkCollectionResponse.statusText}`
      );
    }

    const existingCollection = await checkCollectionResponse.json();

    if (existingCollection.smart_collections.length > 0) {
      collectionResponseData = existingCollection.smart_collections[0];
      collectionExists = true;
    } else {
      // Create the collection if it does not exist
      const collectionResponse = await fetch(collectionUrl, {
        method: "POST",
        headers: {
          "X-Shopify-Access-Token": accessToken,
          "Content-Type": "application/json",
        },
        body: JSON.stringify({ smart_collection: collectionData }),
      });

      if (!collectionResponse.ok) {
        throw new Error(
          `Error adding collection: ${collectionResponse.statusText}`
        );
      }

      const collectionDataResponse = await collectionResponse.json();
      collectionResponseData = collectionDataResponse.smart_collection;
    }
    res.status(200).json({
      message: "Product and Collection processed successfully",
      product: productResponseData,
      productExists: productExists,
      collection: collectionResponseData,
      collectionExists: collectionExists,
    });
  } catch (error) {
    console.error("Failed to add product or collection:", error);
    res.status(500).json({
      error: "Failed to add product or collection",
      details: error.message,
    });
  }
});

app.post("/create-variant", async (req, res) => {
  const shop = req.body.shop;
  const cartId = req.body.cartKey;
  const query = `SELECT accessToken FROM shopify_sessions WHERE shop = "${shop}"`;
  const [rows] = await connection.promise().query(query, [shop]);
  const accessToken = rows.length > 0 ? rows[0].accessToken : null;
  const queryProductId = `SELECT product_id FROM recurrings WHERE shop = "${shop}"`;
  const [rowsProductId] = await connection
    .promise()
    .query(queryProductId, [shop]);
  const productId =
    rowsProductId.length > 0 ? rowsProductId[0].product_id : null;
  try {
    // Fetch existing variants for the product
    const variantsResponse = await axios({
      method: "get",
      url: `https://${shop}/admin/api/${process.env.API_VERSION}/products/${productId}/variants.json`,
      headers: {
        "Content-Type": "application/json",
        "X-Shopify-Access-Token": accessToken,
      },
    });

    const existingVariants = variantsResponse.data.variants;

    // Check if any variant already has the cartId in its metafieldssssss
    for (const variant of existingVariants) {
      const metafieldsResponse = await axios({
        method: "get",
        url: `https://${shop}/admin/api/${process.env.API_VERSION}/variants/${variant.id}/metafields.json`,
        headers: {
          "Content-Type": "application/json",
          "X-Shopify-Access-Token": accessToken,
        },
      });

      const metafields = metafieldsResponse.data.metafields;
      const cartIdMetafield = metafields.find(
        (mf) => mf.key === "cart_id" && mf.value === cartId
      );

      if (cartIdMetafield) {
        await axios({
          method: "put",
          url: `https://${shop}/admin/api/${process.env.API_VERSION}/variants/${variant.id}.json`,
          headers: {
            "Content-Type": "application/json",
            "X-Shopify-Access-Token": accessToken,
          },
          data: {
            variant: {
              id: variant.id,
              price: req.body.variant.price,
            },
          },
        });

        return res
          .status(200)
          .json({ message: "Variant price updated", variant });
      }
    }

    // Create the new variant if no existing one is found
    const createVariantResponse = await axios({
      method: "post",
      url: `https://${shop}/admin/api/${process.env.API_VERSION}/products/${productId}/variants.json`,
      headers: {
        "Content-Type": "application/json",
        "X-Shopify-Access-Token": accessToken,
      },
      data: {
        variant: req.body.variant,
      },
    });

    // Add the cart_id metafield to the newly created variant
    const newVariantId = createVariantResponse.data.variant.id;
    const responseMeta = await axios({
      method: "post",
      url: `https://${shop}/admin/api/${process.env.API_VERSION}/variants/${newVariantId}/metafields.json`,
      headers: {
        "Content-Type": "application/json",
        "X-Shopify-Access-Token": accessToken,
      },
      data: {
        metafield: {
          key: "cart_id",
          value: cartId,
          namespace: "custom",
          type: "single_line_text_field",
        },
      },
    });
    res.status(201).json({
      message: "Variant created successfully",
      variant: createVariantResponse.data.variant,
    });
  } catch (error) {
    console.error(
      "Error creating variant:",
      error.response ? error.response.data : error.message
    );
    res.status(500).json({ error: "Failed to create variant" });
  }
});

app.post("/getVariantToAddToCart", async (req, res) => {
  try {
    const shop = req.body.shop;
    const cartPrice = req.body.cartPrice;

    const query = `SELECT accessToken FROM shopify_sessions WHERE shop = "${shop}"`;
    const [rows] = await connection.promise().query(query, [shop]);
    const accessToken = rows.length > 0 ? rows[0].accessToken : null;
    const queryProductId = `SELECT product_id FROM recurrings WHERE shop = "${shop}"`;
    const [rowsProductId] = await connection
      .promise()
      .query(queryProductId, [shop]);
    const productId =
      rowsProductId.length > 0 ? rowsProductId[0].product_id : null;
    const variantsResponse = await axios({
      method: "get",
      url: `https://${shop}/admin/api/${process.env.API_VERSION}/products/${productId}/variants.json`,
      headers: {
        "Content-Type": "application/json",
        "X-Shopify-Access-Token": accessToken,
      },
    });
    const variants = variantsResponse.data.variants;
    let variantToAdd;
    if (cartPrice <= 40) {
      variantToAdd = variants.filter((variant) => {
        return variant.price === "1.99";
      });
    }
    if (cartPrice > 40 && cartPrice <= 50) {
      variantToAdd = variants.filter((variant) => {
        return variant.title === "2.49";
      });
    }
    if (cartPrice > 50 && cartPrice <= 75) {
      variantToAdd = variants.filter((variant) => {
        return variant.price === "2.99";
      });
    }
    if (cartPrice > 70 && cartPrice <= 100) {
      variantToAdd = variants.filter((variant) => {
        return variant.price === "4.00";
      });
    }
    res.send({ variant: variantToAdd });
  } catch (err) {
    console.log(err, "-------------err getVariantToAddToCart");
    res.status(500).json({ error: "Failed to get variant to add to cart" });
  }
});

app.post("/getvariantToRemove", async (req, res) => {
  const shop = req.body.shop;
  const cartId = req.body.cartId;

  const query = `SELECT accessToken FROM shopify_sessions WHERE shop = "${shop}"`;
  const [rows] = await connection.promise().query(query, [shop]);
  const accessToken = rows.length > 0 ? rows[0].accessToken : null;

  try {
    const queryProductId = `SELECT product_id FROM recurrings WHERE shop = "${shop}"`;
    const [rowsProductId] = await connection
      .promise()
      .query(queryProductId, [shop]);
    const productId =
      rowsProductId.length > 0 ? rowsProductId[0].product_id : null;
    const variantsResponse = await axios({
      method: "get",
      url: `https://${shop}/admin/api/${process.env.API_VERSION}/products/${productId}/variants.json`,
      headers: {
        "Content-Type": "application/json",
        "X-Shopify-Access-Token": accessToken,
      },
    });

    const variants = variantsResponse.data.variants;
    const matchingVariant = [];
    // Iterate over the variants to find the ones with the matching cart ID in their metafields
    for (const variant of variants) {
      const metafieldsResponse = await axios({
        method: "get",
        url: `https://${shop}/admin/api/${process.env.API_VERSION}/variants/${variant.id}/metafields.json`,
        headers: {
          "Content-Type": "application/json",
          "X-Shopify-Access-Token": accessToken,
        },
      });

      const metafields = metafieldsResponse.data.metafields;
      const cartIdMetafield = metafields.find(
        (mf) => mf.key === "cart_id" && mf.value === cartId
      );

      if (cartIdMetafield) {
        matchingVariant.push(variant);
      } else {
      }
    }

    if (matchingVariant.length > 0) {
      res.send({
        status: 200,
        matchingVariant,
      });
    } else {
      res.send({ status: 501, message: "No variants found for this cart ID" });
    }
  } catch (error) {
    console.error(
      "Error fetching variants or metafields:",
      error.response ? error.response.data : error.message
    );
    res.status(500).json({ error: "Failed to fetch variants or metafields" });
  }
});
app.post("/api/recurring", async (req, res) => {
  const shop = res.locals.shopify.session.shop;
  const accessToken = res.locals.shopify.session.accessToken;

  let body;

  if (shop === `${process.env.SHOP_NAME}`) {
    body = {
      recurring_application_charge: {
        name: "Basic Plan",
        price: "0.0",
        return_url: `https://${shop}/admin/apps/${process.env.LIVE_APP_NAME}/`,
        trial_days: 0,
        test: false,
        capped_amount: "20,000",
        terms: "test",
      },
    };
  } else {
    body = {
      recurring_application_charge: {
        name: "Basic Plan",
        price: "0.0",
        return_url: `https://${shop}/admin/apps/${process.env.LIVE_APP_NAME}/`,
        trial_days: 0,
        test: false,
        capped_amount: "20,000",
        terms: "Priority Protection Fee",
      },
    };
  }

  try {
    const response = await axios.post(
      `https://${shop}/admin/api/${process.env.API_VERSION}/recurring_application_charges.json`,
      body,
      {
        headers: {
          "X-Shopify-Access-Token": accessToken,
        },
      }
    );
    res.send({ status: 201, data: response.data });
  } catch (error) {
    res.send({ error: error });
  }
});
app.post("/usageCharge", async (req, res) => {
  const shop = req.body.shop;
  const query = `SELECT accessToken FROM shopify_sessions WHERE shop = "${shop}"`;
  const [rows] = await connection.promise().query(query, [shop]);
  const accessToken = rows.length > 0 ? rows[0].accessToken : null;
  const queryRecurringId = `SELECT charge_id FROM recurrings WHERE shop = "${shop}"`;
  const [rowsRecurrring] = await connection
    .promise()
    .query(queryRecurringId, [shop]);
  const recurringId = rows.length > 0 ? rowsRecurrring[0].charge_id : null;
  try {
    const description = req.body.description;
    const amount = req.body.price;
    const response = await axios.post(
      `https://${shop}/admin/api/${process.env.API_VERSION}/recurring_application_charges/${recurringId}/usage_charges.json`,
      {
       usage_charge: {
          description: "test",
          price: amount,
        },
      },
      {
        headers: {
          "X-Shopify-Access-Token": accessToken,
        },
      }
    );
    res.send({ status: 201, data: response.data });
  } catch (err) {
    console.log(err, "-------------err usage charge");
  }
});
const getRecurringStatusFromDB = async (shop) => {
  const query = `SELECT * FROM recurrings WHERE shop = ?`;
  const [rowsRecurrring] = await connection.promise().query(query, [shop]);
  return rowsRecurrring.length > 0 ? rowsRecurrring[0].status : null;
};

app.get("/getRecurringStatus", async (req, res) => {
  try {
    const shop = req.query.shop;
    const recurringStatus = await getRecurringStatusFromDB(shop);
    res.send({ recurringStatus });
  } catch (error) {
    res.status(500).send({ error: "Failed to fetch recurring status" });
  }
});

app.get("/api/getRecurringStatus", async (req, res) => {
  try {
    const shop = res.locals.shopify.session.shop;
    const recurringStatus = await getRecurringStatusFromDB(shop);
    res.send({ recurringStatus });
  } catch (error) {
    res.status(500).send({ error: "Failed to fetch recurring status" });
  }
});
app.get("/api/getAppId", async (req, res) => {
  const shop = res.locals.shopify.session.shop;
  try {
    res.send({ appId: process.env.APP_ID, shop: shop });
  } catch (err) {
    console.log(err);
  }
});
// ============================route to play video==================================
app.get("/demoVideo", (req, res) => {
  const videoPath = path.join(
    __dirname,
    "frontend",
    "assets",
    "demoVideo.webm"
  );
  const stat = fs.statSync(videoPath);
  const fileSize = stat.size;
  const range = req.headers.range;

  if (range) {
    const parts = range.replace(/bytes=/, "").split("-");
    const start = parseInt(parts[0], 10);
    const end = parts[1] ? parseInt(parts[1], 10) : fileSize - 1;

    const chunksize = end - start + 1;
    const file = fs.createReadStream(videoPath, { start, end });
    const head = {
      "Content-Range": `bytes ${start}-${end}/${fileSize}`,
      "Accept-Ranges": "bytes",
      "Content-Length": chunksize,
      "Content-Type": "video/webm",
    };

    res.writeHead(206, head);
    file.pipe(res);
  } else {
    const head = {
      "Content-Length": fileSize,
      "Content-Type": "video/webm",
    };
    res.writeHead(200, head);
    fs.createReadStream(videoPath).pipe(res);
  }
});
app.get("/privacyPolicy", async (_req, res) => {
  try {
    const __filename = fileURLToPath(import.meta.url);
    const __dirname = dirname(__filename);

    const pdfPath = path.join(
      __dirname,
      "./frontend/assets",
      "priority-protection-privacy-policy.pdf"
    );

    // Set the appropriate content type for PDF
    res.setHeader("Content-Type", "application/pdf");

    // Send the PDF file
    res.sendFile(pdfPath);
  } catch (error) {
    res.send(error);
  }
});
app.use(shopify.cspHeaders());
app.use(serveStatic(STATIC_PATH, { index: false }));

app.use("/*", shopify.ensureInstalledOnShop(), async (_req, res, _next) => {
  return res
    .status(200)
    .set("Content-Type", "text/html")
    .send(readFileSync(join(STATIC_PATH, "index.html")));
});

app.listen(PORT);
